package com.sd.business.service.statement.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.PageUtils;
import com.sd.business.entity.excel.enums.ExcelTypeEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.statement.dto.ExportDocumentDto;
import com.sd.business.entity.statement.dto.GetDocumentDto;
import com.sd.business.entity.statement.dto.SalesOutWarehouseDetailsDto;
import com.sd.business.entity.statement.dto.StatementOfAccountMergePageDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.DocumentByBomVo;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import com.sd.business.entity.statement.vo.DocumentBySkuVo;
import com.sd.business.entity.statement.vo.StatementOfAccountMergePageVo;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.excel.ExcelGenerateLogService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountMergeService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.strategy.impl.DocumentByOrderExcelExportStrategy;
import com.sd.business.strategy.impl.SalesOutWarehouseDetailsExportStrategy;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@Service
public class StatementOfAccountMergeServiceImpl implements StatementOfAccountMergeService {

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private StatementOfAccountExportService statementOfAccountExportService;

    @Autowired
    private ExcelGenerateLogService excelGenerateLogService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Override
    public Page<StatementOfAccountMergePageVo> getPage(StatementOfAccountMergePageDto dto) {

        QueryWrapper<StatementOfAccount> wrapper = Wrappers.query();

        wrapper.eq(ObjectUtil.isNotNull(dto.getDepartmentId()), "department_id", dto.getDepartmentId());
        wrapper.groupBy("department_id");
        wrapper.orderByDesc("dimensionality");

        switch (dto.getType()) {
            // 月度
            case 1:
                wrapper.select("department_id AS departmentId",
                        "GROUP_CONCAT(id) AS idGroupConcat",
                        "DATE_FORMAT(time_period,'%Y/%m月') AS dimensionality");
                wrapper.groupBy("dimensionality");
                break;

            // 季度
            case 2:
                wrapper.select("department_id AS departmentId",
                        "GROUP_CONCAT(id) AS idGroupConcat",
                        "CONCAT(YEAR(time_period),'/第',QUARTER(time_period),'季度') AS dimensionality");
                wrapper.groupBy("dimensionality");
                break;

            // 年度
            case 3:
                wrapper.select("department_id AS departmentId",
                        "GROUP_CONCAT(id) AS idGroupConcat",
                        "YEAR(time_period) AS dimensionality");
                wrapper.groupBy("dimensionality");
                break;

            // 自定义
            case 4:
                String beginTime = dto.getBeginTime() == null ? StringPool.EMPTY : DateUtil.formatDate(dto.getBeginTime());
                String endTime = dto.getEndTime() == null ? StringPool.EMPTY : DateUtil.formatDate(dto.getEndTime());

                wrapper.select("department_id AS departmentId",
                        "GROUP_CONCAT(id) AS idGroupConcat",
                        "'" + beginTime + " - " + endTime + "' AS dimensionality");
                wrapper.ge(ObjectUtil.isNotNull(dto.getBeginTime()), "time_period", dto.getBeginTime());
                wrapper.le(ObjectUtil.isNotNull(dto.getEndTime()), "time_period", dto.getEndTime());
                break;

            default:
                throw new ServiceException("未知对账类型");
        }

        Page<Map<String, Object>> mapPage = statementOfAccountService.pageMaps(dto.getPage(), wrapper);
        Page<StatementOfAccountMergePageVo> page = PageUtils.copyPage(mapPage, StatementOfAccountMergePageVo.class);
        List<StatementOfAccountMergePageVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        departmentService.attributeAssign(records, StatementOfAccountMergePageVo::getDepartmentId,
                (item, department) -> item.setDepartmentName(department.getName()));

        // 对账单id列表
        List<Long> statementOfAccountIdList = records.stream()
                .map(StatementOfAccountMergePageVo::getIdGroupConcat)
                .flatMap(item -> Arrays.stream(item.split(",")))
                .map(Convert::toLong)
                .distinct()
                .collect(Collectors.toList());

        Map<Long, List<OrderInfo>> map = orderService.mapKGroup(OrderInfo::getStatementOfAccountId,
                q -> q.in(OrderInfo::getStatementOfAccountId, statementOfAccountIdList));

        for (StatementOfAccountMergePageVo record : records) {

            String[] tempStatementOfAccountIdList = record.getIdGroupConcat().split(",");

            record.setReconcilingAmount(BigDecimal.ZERO);
            record.setOrderQuantity(0);
            record.setQuantityOfStatement(tempStatementOfAccountIdList.length);

            for (String statementOfAccountIdStr : tempStatementOfAccountIdList) {
                Long statementOfAccountId = Convert.toLong(statementOfAccountIdStr);
                List<OrderInfo> tempOrderInfoList = map.get(statementOfAccountId);

                if (ObjectUtil.isEmpty(tempOrderInfoList)) {
                    continue;
                }

                BigDecimal reconcilingAmount = tempOrderInfoList.stream()
                        .map(OrderInfo::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

                record.setOrderQuantity(record.getOrderQuantity() + tempOrderInfoList.size());
                record.setReconcilingAmount(record.getReconcilingAmount().add(reconcilingAmount));
            }

        }

        return page;
    }

    @Override
    public List<DocumentBySkuVo> getDocumentBySku(GetDocumentDto dto) {
        return statementOfAccountService.getSkuDocument(getIdList(dto));
    }

    @Override
    public List<DocumentByBomVo> getDocumentByBom(GetDocumentDto dto) {
        return statementOfAccountService.getBomDocument(getIdList(dto));
    }

    @Override
    public List<DocumentByOrderVo> getDocumentByOrder(GetDocumentDto dto) {
        return statementOfAccountService.getOrderDocument(getIdList(dto));
    }

    @Override
    public void export(ExportDocumentDto dto, int type) {

        List<Long> idList = Arrays.stream(dto.getIdGroupConcat().split(","))
                .map(Convert::toLong)
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());

        String departmentName = dto.getDepartmentName();
        String beginDate = dto.getBeginDate();
        String endDate = dto.getEndDate();

        switch (type) {
            case 1:
                statementOfAccountExportService.exportDocumentBySku(idList, departmentName, beginDate, endDate);
                break;
            case 2:
                statementOfAccountExportService.exportDocumentByBom(idList, departmentName, beginDate, endDate);
                break;
            case 3:
                excelGenerateLogService.generateExcel(ExcelTypeEnum.STATISTICS_DOCUMENT_BY_ORDER,
                        new DocumentByOrderExcelExportStrategy(idList, departmentName, beginDate, endDate),
                        DateUtil.formatDate(new Date()) + " 对账报表");
                break;
            default:
                throw new ServiceException("未知对账单类型");
        }
    }

    @Override
    public Page<DocumentByOrderVo> salesOutWarehouseDetails(SalesOutWarehouseDetailsDto dto) {

        // 对账单条件查询
        List<StatementOfAccount> statementOfAccountList = null;
        List<Long> statementOfAccountIdList = null;

        if (!ObjectUtil.isAllEmpty(dto.getBeginTime(), dto.getEndTime())) {
            statementOfAccountList = statementOfAccountService.list(q -> q
                    .ge(ObjectUtil.isNotNull(dto.getBeginTime()), BasePo::getCreateTime, dto.getBeginTime())
                    .le(ObjectUtil.isNotNull(dto.getEndTime()), BasePo::getCreateTime, dto.getEndTime()));

            if (statementOfAccountList.isEmpty()) {
                return new Page<>();
            }
            statementOfAccountIdList = statementOfAccountList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        }

        // sku条件查询
        List<Long> orderIdList = null;
        if (!StrUtil.isAllBlank(dto.getSkuSpecCode(), dto.getSkuSpecName())) {
            List<SkuSpec> skuSpecList = skuSpecService.list(q -> q
                    .like(StrUtil.isNotBlank(dto.getSkuSpecCode()), SkuSpec::getCode, dto.getSkuSpecCode())
                    .like(StrUtil.isNotBlank(dto.getSkuSpecName()), SkuSpec::getName, dto.getSkuSpecName()));

            if (skuSpecList.isEmpty()) {
                return new Page<>();
            }
            List<Long> skuSpecIdList = skuSpecList.stream().map(BaseIdPo::getId).collect(Collectors.toList());

            List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getSkuSpecId, skuSpecIdList));
            if (orderSkuList.isEmpty()) {
                return new Page<>();
            }

            orderIdList = orderSkuList.stream().map(OrderSku::getOrderId).collect(Collectors.toList());
        }

        Page<OrderInfo> page = dto.getPage();
        page.setSearchCount(dto.getSearchCount());

        // 查询订单分页
        orderService.page(page, Wrappers.lambdaQuery(OrderInfo.class)
                .in(ObjectUtil.isNotEmpty(orderIdList), BaseIdPo::getId, orderIdList)
                .in(ObjectUtil.isNotEmpty(statementOfAccountIdList), OrderInfo::getStatementOfAccountId, statementOfAccountIdList)
                .isNotNull(ObjectUtil.isEmpty(statementOfAccountIdList), OrderInfo::getStatementOfAccountId)
                .eq(ObjectUtil.isNotNull(dto.getDepartmentId()), OrderInfo::getDepartmentId, dto.getDepartmentId())
                .and(StrUtil.isNotBlank(dto.getCode()), q -> q
                        .like(OrderInfo::getCode, dto.getCode()).or().like(OrderInfo::getWlnCode, dto.getCode()))
                .orderByDesc(OrderInfo::getStatementOfAccountId)
        );

        List<OrderInfo> records = page.getRecords();

        if (records.isEmpty()) {
            return new Page<>();
        }

        // 赋值订单sku，包材
        List<DocumentByOrderVo> orderDocumentByOrderList = statementOfAccountService.getOrderDocumentByOrderList(records);

        // 赋值事业部名称
        departmentService.attributeAssign(orderDocumentByOrderList, DocumentByOrderVo::getDepartmentId, (item, department) -> {
            item.setDepartmentName(department.getName());
        });

        // 赋值对账时间
        if (statementOfAccountList == null) {
            statementOfAccountList = statementOfAccountService.listByIds(
                    records.stream().map(OrderInfo::getStatementOfAccountId).collect(Collectors.toList()));
        }
        Map<Long, StatementOfAccount> map = statementOfAccountList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        for (DocumentByOrderVo documentByOrderVo : orderDocumentByOrderList) {
            Long statementOfAccountId = documentByOrderVo.getStatementOfAccountId();
            StatementOfAccount statementOfAccount = map.get(statementOfAccountId);
            documentByOrderVo.setStatementOfAccountTime(statementOfAccount.getCreateTime());
        }

        Page<DocumentByOrderVo> result = new Page<>();
        result.setTotal(page.getTotal());
        result.setRecords(orderDocumentByOrderList);
        result.setPages(page.getPages());
        result.setSize(page.getSize());

        return result;
    }

    @Override
    public void exportSalesOutWarehouseDetails(SalesOutWarehouseDetailsDto dto) {

        excelGenerateLogService.generateExcel(ExcelTypeEnum.SALES_OUT_WAREHOUSE_DETAILS,
                new SalesOutWarehouseDetailsExportStrategy(this, dto),
                DateUtil.formatDate(new Date()) + " 销售出货明细");
    }

    private List<Long> getIdList(GetDocumentDto dto) {

        String idGroupConcat = dto.getIdGroupConcat();

        List<Long> idList = Arrays.stream(idGroupConcat.split(","))
                .map(Convert::toLong)
                .distinct()
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        Assert.notEmpty(idList, "对账单id集合不能为空");

        return idList;
    }

}
